Puppet::Functions.create_function(:'netinfo::get_outside_ipv4_addr') do
	dispatch :get_outside_ipv4_addr do
	end

	def get_outside_ipv4_addr
		#local variables
        scope=closure_scope.to_hash
        #try to determine outside interfaces
        unless (outside_if=call_function('lookup','network::outside_interface',Puppet::Pops::Types::PStringType::DEFAULT,'first','') and !outside_if.empty?)
          unless (outside_if=call_function('lookup','network::dmz',Puppet::Pops::Types::PStringType::DEFAULT,'first','') and !outside_if.empty?)
            fail("network::nat44: outgoing interface could not be determined, set network::outside_interface or network::dmz")
          end
        end
        network_config=call_function('lookup','network::ipv4',Puppet::Pops::Types::PHashType::DEFAULT,'hash',{})
        # get local IPv4 address:
        # - check if config contains 'v4_address'-field
        # - check for service address in network::carp::ipv4::address
        # - if this fails, use first address in facts/network/#{dmz_if}/bindings
        # - if this too fails, fail this function as NAT44 without a local IPv4 address does not make sense
        if (network_config.key?(outside_if) and network_config[outside_if].key?('address'))
          my_addr=network_config[outside_if]['address']
        else
          failover_config=call_function('lookup','network::failover',Puppet::Pops::Types::PHashType::DEFAULT,'hash',{})
          if !failover_config.empty? \
            and failover_config.key?(outside_if) \
            and failover_config[outside_if].key?('ipv4') \
            and failover_config[outside_if]['ipv4'].key?('addresses') \
            and failover_config[outside_if]['ipv4']['addresses'].size>0
            my_addr=failover_config[outside_if]['ipv4']['addresses'][0]
          else
            unless my_addr=scope.dig('facts','networking','interfaces',outside_if,'bindings',0,'address')
              fail("network::nat44: could not determine local IPv4 address, set network::failover/ipv4/addresses, network::nat44/v4_address or check why fact network/#{outside_if}/bindings is missing")
            end
          end
        end
        my_addr.gsub(/\/[0-9]*/,'')
    end
end
