Puppet::Functions.create_function(:'netinfo::get_outside_ipv6_addrs') do
    dispatch :get_outside_ipv6_addrs do
    end

    def get_outside_ipv6_addrs
      #local variables
      addrs=[]
      scope=closure_scope.to_hash
      #try to determine outside interfaces
      unless (outside_if=call_function('lookup','network::outside_interface',Puppet::Pops::Types::PStringType::DEFAULT,'first','') and !outside_if.empty?)
        unless (outside_if=call_function('lookup','network::dmz',Puppet::Pops::Types::PStringType::DEFAULT,'first','') and !outside_if.empty?)
          fail("outgoing interface could not be determined, set network::outside_interface or network::dmz")
        end
      end
      network_config=call_function('lookup','network::ipv6',Puppet::Pops::Types::PHashType::DEFAULT,'hash',{})
      # get local IPv6 address:
      # - check if config contains 'v6_address'-field
      # - check for service address in network::failover
      # - if this fails, use first address in facts/network/#{dmz_if}/bindings
      # - if this too fails, fail this function as NAT44 without a local IPv6 address does not make sense
      if network_config.key?(outside_if) and network_config[outside_if].size>0
        addrs+=network_config[outside_if]
      end
      failover_config=call_function('lookup','network::failover',Puppet::Pops::Types::PHashType::DEFAULT,'hash',{})
      if !failover_config.empty? \
        and failover_config.key?(outside_if) \
        and failover_config[outside_if].key?('ipv6') \
        and failover_config[outside_if]['ipv6'].key?('addresses') \
        and failover_config[outside_if]['ipv6']['addresses'].size>0
        addrs+=failover_config[outside_if]['ipv6']['addresses']
      end
      configured_bindings=scope.dig('facts','networking','interfaces',outside_if,'bindings6')
      configured_bindings.each do|binding|
        if binding['address']!~/^fe80:/
          addrs << binding['address']
        end
	  end
	  #remove duplicate addresses
      unless addrs.size>0
        fail("network::nat44: could not determine local IPv6 address, set network::failover/ipv6/addresses, network::nat44/v6_address or check why fact network/#{outside_if}/bindings is missing")
      end
	  addrs.map{|a| call_function('netinfo::reduce_ipv6',a.gsub(/\/[0-9]+/,'')).gsub(/\/[0-9]+/,'')}.uniq
    end
end
