Puppet::Functions.create_function(:'netinfo::validate_ipv6') do
  dispatch :validate_ipv6 do
    param 'String', :prefix
  end

  def validate_ipv6(prefix_in)
    prefix = prefix_in.dup #otherwise frozen prefix_in causes failure
    prefix.downcase!
    if %r{^(?<addr>[a-f0-9:]+)(\/(?<len>[0-9]{1,3}))?$} =~ prefix
      if len
        len = len.to_i
        if len<0 || len>128
          #prefix length invalid
          return false
        end
      end
      addr = addr.split(':')
      if addr.size == 9 && addr.include?('')
        #9 blocks -> there might be one useless :: -> if so, remove it, otherwise fail
        addr.delete('')
      end
      if addr.size > 8
        #more than 8 blocks
        return false
      end
      empty_seen = false
      first_block = true #if the first block is empty, empty_seen is set in the first iteration though it should not be set
      addr.each do |a|
        if a.empty?
          if empty_seen
            return false
          end
          empty_seen = true unless first_block
        end
        if a.size > 4
          #one block with more than 4 chars
          return false
        end
        first_block = false
      end
      true
    else
      #no IPv6 address
      false
    end
  end
end
